<div class="card" id="page-toc-wrapper">
  <div class="card-header">
    <h4>{% trans %}In this page:{% endtrans %}</h4>
  </div>
  <ul id="page-toc" class="list-group card-body" role="directory">
    {% for h in headers %}
    <li class="level-{{h.level}}"><a class="nav-link" href="#{{h.id}}">{{h.text}}</a></li>
    {% endfor %}
  </ul>
</div>
