{% extends 'base.html' %}

{% block main %}
<article class="pt-3 p-md-3">
  <h1>Not Found</h1>
  <div class="content">
      <p>Sorry, this page does not exist.</p>
  </div>
</article>
{% endblock %}
