{% extends "template-base.html" %}
{% block head %}

<meta http-equiv="refresh" content="0;url={{currentpage.redirect_url}}" />
{% endblock %}


{% block main %}
  <article class="pt-3 p-md-3">
    <div class="content">
        <p>This page has been moved! You should be redirected automatically. If not, <a href="{{target.prefix}}{{currentpage.redirect_url}}">click here to go to the new {{currentpage.name}} page</a>.
    </div>
  </article>
{% endblock %}
